'***** AXE114 Binary Clock Program *****

#picaxe 28x

'***** Input/Output Pins *****

'- Output LEDs
'output7		min32		-
'output6		min16		day16
'output5		min8		day8
'output4		min4		day4
'output3		min2		day2
'output2		min1		day1
'output0		hours1	month1				
'outputc7		hours2	month2
'outputc6		hours4	month4
'outputc5		hours8	month8

' - Other outputs
'output1		(optional piezo sounder PZ)
'outputc2		transistor 2
'outputc1		transistor 1

'- Inputs
'input4		DS1307 SDA
'input3		DS1307 SCL
'input0		switch A (S1)
'analogue 0		LDR
'porta pin1		switch < (S3)
'porta pin2 	switch > (S2)
'analogue 3		not used

'***** Variables *****
symbol mins = b0			' store for minutes value
symbol hours = b1			' store for hours value
symbol days = b2			' store for days value
symbol months = b3		' store for months value
symbol light_value  = b4	' store for LDR value
symbol temp = b5			' temporary
symbol temp_port = b6		' temporary port value


'***** Initialisation *****
init:
	' setup portC
	' as some inputs are changed to be outputs
	let dirsc = %11100110
	
	' setup i2c for DS1307 clock  operation
	i2cslave %11010000, i2cslow, i2cbyte

'***** Main Loop *****	
main:
	' read current time from DS1307
	readi2c 0,(temp,mins,hours,temp,days,months)
	'change clock values from BCD to binary
	gosub	bcd_bin
	'optionally display on computer via debug
	'debug
	
	'display the hours and minutes
	gosub display_hours_mins
	
	'display the days and months
	gosub display_days_months
	
	goto main


'***** Sub Procedure - convert BCD to binary *****
bcd_bin:
	' convert the DS1307 BCD values to binary
	temp = mins AND %00001111
	mins = mins AND %11110000 * 10 / 16 + temp
	
	temp = hours AND %00001111
	hours = hours AND %11110000 * 10 / 16 + temp
	
	temp = days AND %00001111
	days = days AND %11110000 * 10 / 16 + temp
	
	temp = months AND %00001111
	months = months AND %11110000 * 10 / 16 + temp

	' convert 24 hour clock values to 12 hour clock
	if hours < 13 then do_return
	hours = hours - 12
	
do_return:
	return

'***** Sub Procedure - display the hours/mins *****

display_hours_mins:
	' display hours and minutes
	let temp_port = mins * 4 AND %11111100
	let temp = hours AND %00000001
	if temp = 0 then hm1
	let temp_port = temp_port OR %00000001	
hm1:
	let pins = temp_port

	let pinsc = 0
	let temp= hours AND %00000010
	if temp = 0 then hm2
	high portc 7	
hm2:
	let temp= hours AND %00000100
	if temp = 0 then hm3
	high portc 6	
hm3:
	let temp= hours AND %00001000
	if temp = 0 then hm4
	high portc 5	
hm4:
	' now actually light LEDs for a while
	' by enabling the correct transistor
	high portc 2
	pause 2000
	low portc 2
	return

'***** Sub Procedure - display the days/months *****

display_days_months:
	' display days and months	
	let temp_port = days * 4 AND %01111100
	let temp = months AND 1
	if temp = 0 then dm1
	let temp_port = temp_port OR %00000001	
dm1:
	let pins = temp_port
	
	let pinsc = 0
	let temp = months AND %00000010
	if temp = 0 then dm2
	high portc 7	
dm2:
	let temp= months AND %00000100
	if temp = 0 then dm3
	high portc 6	
dm3:
	let temp= months AND %00001000
	if temp = 0 then dm4
	high portc 5	
dm4:
	' now actually light LEDs for a while
	' by enabling the correct transistor
	high portc 1
	pause 2000
	low portc 1
	
	return
	
	
'***** Extra Information *****

' Examples on how to use the other inputs/outputs	

' to react to LDR light sensor
'	readadc 0,light_value
'	if light_value > 80 then...

' to make a sound on piezo
'	sound 1,(50,50)

' to read the three switches
'	if pin0 = 1 then...
'	if porta pin0 = 1 then...
'	if porta pin1 = 1 then...



