#picaxe 28x2
#no_table
#no_data

;Power supply Jumper J1 options
;Note the 5V logic supply (e.g. for inputs) always comes from base

; Off 	Shield power terminal is used for outputs only
;		Base must use its own power supply
; Right	Base power supply is used for everything, 5V supply to outputs
;		No power connection should be made to shield power terminal
; Left	Shield power connects to base VIn so either
;		- shield power only, also supplies base via Vin
;		- base power only, full Vin is applied to shield outputs	

;Buffered outputs
; Buff 12 is S.12		1 = on, 0 = off
; Buff 13 is S.13		1 = on, 0 = off

;Motors
; Motor A
; Direction is S.8	1 = one way, 0 = the other
; On/Off/PWM is S.9	1 = on, pulsing=pwm, 0 = off

; Motor B
; Direction is S.11	1 = one way, 0 = the other
; On/Off/PWM is S.10	1 = on, pulsing=pwm, 0 = off

; PWM frequency is critical for good motor control
; and varies between motor types
; for instance 20% at 10kHz works, but 20% at 1kHz just 'hums'

; pwmout S.9, 99, 200			; 50% at 10kHz
; pwmout S.9, 99, 80			; 20% at 10kHz
; pwmout pwmdiv4, S.9, 249, 500	; 50% at 1kHz
; pwmout pwmdiv4, S.9, 249, 200	; 20% at 1kHz
; Good noise suppression is also essential on the motor

;Input Pins 
; S.A0 to S.A5 are all digital inputs (if pinA.0 = 1 then)
; Can also be analogue inputs (readadc S.A0, b1)
; Can also be touch sensor inputs (touch S.A0, b1)
; Each has an optional jumper which connects a 10k pull down resistor

;General Input/Outputs
; S.0 to S.7 are general purpose outputs (or inputs)
; S.2 to S.7 can be servos (e.g. servo S.2,75)
; S.0 and S.1 can be hardware serial port (e.g. hserin/hserout)

;Ultrasonic
; Ping or SRF05 can be used on S.5

;Infrared LED
; IR transmitter can be used on S.6 (e.g. irout S.6,1,1)

;Infrared Sensor
; IR sensor can be used on S.7 (e.g. irin S.7, b1)


; Test program
; Activate an inputy (short signal to V+)
; To activate motors, outputs etc.

init:
	low S.13		; buffer 13 off
	low S.12		; buffer 12 off
	low S.10		; motor off
	low S.9		; motor off
	servo S.7, 200	; servo on S.7 to position 200

do

	if pinS.A0 = 1 then
		high S.8	; motor forward
		high S.9	; motor on
	else
		low S.9	; motor off
	end if

	if pinS.A1 = 1 then
		high S.11	; motor forward
		high S.10	; motor on
	else
		low S.10	; motor off
	end if


	if pinS.A2 = 1 then
		high S.12	; buffer output on
	else
		low S.12	; buffer output off
	end if

	if pinS.A3 = 1 then
		high S.13	; buffer output on
	else
		low S.13	; buffer output off
	end if


	if pinS.A4 = 1 then
		servopos S.7, 100	; move servo
	else
		servopos S.7,200	; move servo
	end if
	
loop


