'PROGRAM: ADC002.BAS

'BASIC Stamp program that demonstrates
'how to use the serial ADC module with 
'the serial LCD module.

'Every second the ADC performs a reading
'and displays the ADC value on the LCD.

'Connect the ADC module to the Stamp, 
'and then connect a sensor to the ADC.
'Connect the LCD module to the Stamp.

symbol	ADC_CS	=	0	'ADC chip select pin (output)
symbol	ADC_CLK	=	1	'ADC clock pin (output)
symbol	ADC_D_I	=	pin2	'ADC data pin (input)
symbol 	switch	= 	pin3	'ADC push switch (input)

symbol	S_OUT	= 	7	'serial LCD output (output)

symbol	data	=	b4	'data byte
symbol	i	=	b5	'scratchpad counter
symbol	counter = 	b10	'counter

' Initialise pins

init:	let dirs = %11110011	' pin 2 & 3 input, rest outputs. 
	let pins = %10000001	' ADC_CS and S_OUT high

	pause 5				'Short pause
	serout S_OUT,T2400,(254,1) 	'Clear LCD
	pause 30			'Short pause.

' Main program loop

loop:	gosub adcread			'Get the ADC reading.

	'Print counter value on first LCD line
	serout S_OUT,T2400,(254,128,"count= ",#counter,"  ")

	'Print ADC value on second LCD line
	serout S_OUT,T2400,(254,192,"value= ",#data,"  ")

	counter = counter + 1		'Increment counter.
	pause 1000			'Pause for 1 second.
	goto loop			'Loop.
					
' This sub-procedure gets a reading from the serial ADC unit.
' Data is returned in byte variable 'data'

adcread:
	low ADC_CLK			'Put clock line in starting state. 
	low ADC_CS			'Select ADC. 
	pulsout ADC_CLK, 1		'10 us clock pulse. 
	let data = 0			'Clear data. 
	for i = 1 to 8			'Eight data bits. 
	   let data = data * 2		'Perform shift left. 
	   pulsout ADC_CLK, 1		'10 us clock pulse. 
	   let data = data + ADC_D_I	'Put bit in LSB of data. 
	next i				'Do it again. 
	high ADC_CS			'Deselect ADC when done. 
	return	


