
#rem

2008-12-08 modification for V1.3: some pin assignments changed; added def's for direct internal ADC access to frontend
2008-08-24 added support for scope version 1.1 (does hserial through MAX232)
2008-05-12 modified clock set_clock_frequency
2008-05-02 corrected readout - pulsout must come AFTER SRAM read, not before
2008-04-24 merged CMD_TRIG_LVL and CMD_OFFSET to CMD_SET_DAC; added function set_dac
2008-04-24 adding DAC control
2008-04-24 converted SPI bitbanging to spuout command; changed pin assignments to put SPI pins on output port
2008-04-22 readback - added option to read back only a fraction of the record
2008-04-22 added conditional compile (hserout vs. sertxd); added serin vs. sertxd
2008-04-21 acknowledge is now a checksum (where possible); FIRST BYTE OF READBACK IS NOW CHECKSUM AS WELL
2008-04-18 started to add command control
2008-04-15 adding support for PGA; moving outputs to I/O expander; named constants
2008-04-14 partially unrolled readout loop for higher speed
2008-04-12 debugging trigger problem; removed unnecessary legacy pause 400
2008-04-12 single-channel version
2008-04-12 replaced sertxd with hserout
2008-04-12 added slow clock ANDed together with PWM
2008-04-11 handshaking with PC controller software

#endrem


'===============================================
#picaxe 28X1 		' target is picaxe-28x1
;#define HSERIAL		' use hardware serial port - jumper in TX position
#define NO_MAX202		' uses resistor R26 not MAX202CPE chip
'===============================================


symbol REVISION       = 11 ' revision = V1.1



symbol CMD_RESET          = 0  ' resets the controller

symbol CMD_ARM            = 1  ' resets counter, waits for trigger
symbol CMD_AUTOTRIG       = 2  ' forces a trigger event
symbol CMD_DONE           = 3  ' queries "done" status
symbol CMD_READBACK       = 4  ' initiates readback of acquired record

symbol CMD_READADC        = 5  ' reads back ADC directly

symbol CMD_TRIG_POL       = 10 ' sets trigger polarity

symbol CMD_GAIN           = 12 ' sets the PGA gain

symbol CMD_SET_DAC        = 13

symbol CMD_PRE_GAIN       = 14 ' selects pre-amp gain (1 or 10)

symbol CMD_SAMPLE_CLK     = 15 ' sets the sample clock frequency & prescaler

symbol CMD_SET_CLOCK_FREQ = 16

symbol CMD_WRITE_DATA     = 30 ' writes data to internal EEPROM
symbol CMD_READ_DATA      = 31 ' read data from internal EEPROM

symbol CMD_PING           = 100 ' requests echo from controller
symbol CMD_REVISION       = 101 ' requests firmware revision number from controller


symbol CLOCK_16MHz        = 0
symbol CLOCK_8MHz         = 1
symbol CLOCK_4MHz         = 2
symbol CLOCK_2MHz         = 3
symbol CLOCK_1MHz         = 4
symbol CLOCK_500kHz       = 5
symbol CLOCK_250kHz       = 6


symbol READ_CH1           = %00000001
symbol READ_CH2           = %00000010
symbol READ_CH1_CH2       = %00000011


symbol COUNTER2_SETUP_REG = 0x12

symbol IOEXP_CONFIG       = 0x04
symbol IOEXP_PORT_CONFIG  = 0x0b
symbol IOEXP_PGA1_CSn     = 0x2c ' output port 12
symbol IOEXP_PGA2_CSn     = 0x2d ' output port 13
symbol IOEXP_DAC1_CSn     = 0x3e ' output port 30
symbol IOEXP_DAC2_CSn     = 0x3f ' output port 31
symbol IOEXP_SRAM_CH1     = 0x4e ' input ports 14 - 21
symbol IOEXP_SRAM_CH2     = 0x56 ' input ports 22 - 29


' Output pins

symbol TRIG_PICKOFF  = 0
symbol CLK_SLOW      = 1
symbol SPI_CLK       = 2
symbol SPI_DATA      = 3
symbol DFF_TRIG_CLRn = 4
symbol ADC_PWRDNn    = 5
symbol DFF_CTR_CLRn  = 6
symbol DFF_TRIG_PREn = 7


' I/O pins (portc)

symbol LED_portc         = 0
symbol FAST_CLK_portc    = 1
symbol TRIG_POL_portc    = 2
' 3 = I2C SCL
' 4 = I2C SDA
symbol CTR_CLR_portc     = 5
' 6 = hserout
' 7 = not used (hserin capable)

symbol FAST_CLK_PWM_CHAN = 1


' ADC / Input pins (porta)

symbol ADC_DONE = 0
symbol ADC10_1  = 1 ' direct connection to analog frontend
symbol ADC10_2  = 2 ' direct connection to analog frontend
' 3 not used


' Variables

symbol b_pga_instruction = b0
symbol b_pga_data        = b1
symbol b_pga_channel     = b2

symbol b_adc_data        = b3

symbol b_counter         = b4

symbol b_checksum        = b5

symbol b_dac_data_1      = b6
symbol b_dac_data_2      = b7

symbol b_pwm_period      = b19
symbol w_pwm_duty        = w10 ' b20, b21
symbol b_pwm_prescaler   = b22

symbol b_command         = b24
symbol b_param1          = b25
symbol b_param2          = b26

symbol b_dummy           = b27

gosub init



'disconnect
'high portc 6
'myloop:
'
'	serin 7, T19200_16, b0
'	'b0 = b0 + 1
'	'sertxd (#b0, cr, lf)
'	'hserout 0, (#b0, cr, lf)
'	'hserout 0, (b0, " = ", #b0, cr, lf)
'	sertxd (b0, " = ", #b0, cr, lf)
'	'pause 2000
'	
'goto myloop




main:

	'#ifdef HSERIAL			
	'serin 7, T19200_16, b_command, b_param1, b_param2 ' serin goes through MAX232 which inverts the data
	'#else
	serrxd b_command, b_param1, b_param2
	'#endif
	
	b_checksum = b_command xor b_param1 xor b_param2
	
	select b_command

		case CMD_ARM ' params: none

			gosub arm_setup
		
		case CMD_DONE ' params: none

			gosub query_done
			
		case CMD_READBACK ' params: par1 %000000ba  a=0/1 = CH1 no/yes, b=0/1 = CH2 no/yes; par 2: 0=128pts, 1=256pts
		
			gosub readback

		case CMD_SET_CLOCK_FREQ ' par1 = frequeny setting

			gosub set_clock_frequency

		case CMD_AUTOTRIG ' params : none

			gosub force_trigger

		case CMD_READADC ' params: none
'tbd

		case CMD_TRIG_POL ' params: par1 0 = rising, 1 = falling

			gosub set_trigger_polarity
			
		case CMD_SET_DAC ' params: %ddddvvvv %vvvvvvvv: d = DAC channel (0..3), v = DAC value 0..4095

			gosub set_dac

		case CMD_GAIN ' params: par1 = channel, par2 = gain 0..7 = 1/2/4/5/8/10/16/32

			gosub set_gain

		case CMD_PRE_GAIN ' params: par1 0 = DC, 1 = AC

			gosub set_preamp_gain

		case CMD_SAMPLE_CLK ' params: par1 = PWM period, par2 = prescaler (00 = div 1, 01 = div 4, 10 or 11 = div 16)

			gosub set_sample_clock
			
		case CMD_WRITE_DATA ' params: par1 = address, par2 = data

			write b_param1, b_param2

			#ifdef HSERIAL			
			hserout 0, (b_checksum) ' acknowledge
			#else
			sertxd (b_checksum)
			#endif
					
		case CMD_READ_DATA ' params: par1 = address, par2 = data

			read b_param1, b_param2 ' overwrites b_param2

			#ifdef HSERIAL
			hserout 0, (b_param2)
			#else
			sertxd (b_checksum)
			#endif

		case CMD_PING ' params: none

			#ifdef HSERIAL
			hserout 0, ("LCS1M")
			#else
			sertxd ("LCS1M")
			#endif

		case CMD_REVISION ' params: none

			#ifdef HSERIAL
			hserout 0, (REVISION)
			#else
			sertxd (REVISION)
			#endif
			
		case CMD_RESET ' params: none

			reset
			
	endselect

goto main


init:

	setfreq em16

	#ifdef NO_MAX202
	hsersetup B19200_16, %10 ' invert data; not background receive
	#else
	hsersetup B19200_16, %00 ' don't invert data; not background receive
	#endif

	'let dirsc = %00100001

	' initialize I/O expander
	i2cslave %10000000, i2cfast16, i2cbyte ' AN0=GND, AN1=GND - clocked within spec to 400 kHz
	'i2cslave %10000000, i2cfast8, i2cbyte ' AN0=GND, AN1=GND - OVERCLOCKED to 800 kHz!!!
	writei2c IOEXP_CONFIG,(0x01) 'wakeup from shut down 
	' ports 12, 13, 30, 31 are outputs, all other are inputs without pullup
	writei2c IOEXP_PORT_CONFIG,(0xa5,0xaa,0xaa,0xaa,0x5a)

	low portc TRIG_POL_portc ' trigger on rising edge

	' disable databus for both PGA
	writei2c IOEXP_PGA1_CSn,(1)
	writei2c IOEXP_PGA2_CSn,(1)
	writei2c IOEXP_DAC1_CSn,(1)
	writei2c IOEXP_DAC2_CSn,(1)

	low SPI_CLK
	low SPI_DATA
	
	high portc LED_portc
		
return


arm_setup:

	' power up ADCs (high ADC_PWRDNn also means low CSn)
	high ADC_PWRDNn 

	' disable trigger and clock path by forcing the trigger DFF to low
	high DFF_TRIG_PREn : low DFF_TRIG_CLRn
	
	' clear counter loopback DFF
	low DFF_CTR_CLRn : high DFF_CTR_CLRn

	' reset counter to zero
	high portc CTR_CLR_portc : low portc CTR_CLR_portc

	' set up PWM clock
	pwmout FAST_CLK_PWM_CHAN, b_pwm_period, w_pwm_duty

	' slow down PWM if necessary, using counter 2 prescaler
	peek COUNTER2_SETUP_REG, b_dummy
	b_dummy = b_dummy & %11111100 | b_pwm_prescaler ' 00 = div 1, 01 = div 4, 10 or 11 = div 16
	poke COUNTER2_SETUP_REG, b_dummy

	' enable fast PWM clock
	high CLK_SLOW

	' now release trigger DFF to enable trigger
	high DFF_TRIG_CLRn

	#ifdef HSERIAL
	hserout 0, (b_checksum) ' acknowledge
	#else
	sertxd (b_checksum)
	#endif

return


set_preamp_gain:

	b_pga_channel = b_param1
	b_pga_instruction = %01000001 ' select channel
	b_pga_data = b_param2 ' 0 = CH0, 1 = CH1

	gosub pga_command

	#ifdef HSERIAL
	hserout 0, (b_checksum) ' acknowledge
	#else
	sertxd (b_checksum)
	#endif
	
return


force_trigger:

	low DFF_TRIG_PREn
	pause 1
	high DFF_TRIG_PREn

	#ifdef HSERIAL
	hserout 0, (b_checksum) ' acknowledge
	#else
	sertxd (b_checksum)
	#endif

return			


query_done:

	if porta pin0 = 0 then
		#ifdef HSERIAL
		hserout 0, (0)
		#else
		sertxd (0)
		#endif
		
	else

		#ifdef HSERIAL
		hserout 0, (0xff)
		#else
		sertxd (0xff)
		#endif

	endif

return


set_trigger_polarity:

	if b_param1 = 0 then
		low portc TRIG_POL_portc ' trigger on rising edge
	else
		high portc TRIG_POL_portc ' trigger on rising edge
	endif			

	#ifdef HSERIAL
	hserout 0, (b_checksum) ' acknowledge
	#else
	sertxd (b_checksum)
	#endif

return


set_sample_clock:

	b_pwm_period = b_param1
	w_pwm_duty = 2 * b_pwm_period + 2 ' to make clock 50% duty cycle
	b_pwm_prescaler = b_param2 & %00000011
			
	#ifdef HSERIAL
	hserout 0, (b_checksum) ' acknowledge
	#else
	sertxd (b_checksum)
	#endif

return


readback:

	#ifdef HSERIAL
	hserout 0, (b_checksum)
	#else
	sertxd (b_checksum)
	#endif
	
	' stop PWM signal (readout need to be clocked manually)
	pwmout FAST_CLK_PWM_CHAN off
	
	' stop slow clock (if running at slow sample rate)
	settimer off

	' ADC powerdown (outputs become high impedance, so SRAM can take over and be read)
	low ADC_PWRDNn

	' clear counter DFF, set trigger DFF --> enables clock to counter
	low DFF_CTR_CLRn ' counter DFF
	low DFF_TRIG_PREn ' trigger DFF

	' stop PWM clock and enable slow (readout) clock
	high portc FAST_CLK_portc
	low CLK_SLOW

	' counter loop partially unrolled to reduce loop overhead, maximize readout speed

	' full (1) or half (0) record
	if b_param2 = 1 then
	
		if b_param1 = READ_CH1 then
	
			for b_counter = 0 to 31
	
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
			next b_counter
	
		else if b_param1 = READ_CH2 then
		
			for b_counter = 0 to 31
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
			next b_counter
	
		else if b_param1 = READ_CH1_CH2 then
		
			for b_counter = 0 to 31
	
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1

			next b_counter

		endif

	else ' param2 = 0 --> send only every other sample
	
		if b_param1 = READ_CH1 then
	
			for b_counter = 0 to 15
	
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
			next b_counter
	
		else if b_param1 = READ_CH2 then
		
			for b_counter = 0 to 15
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
		
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
			next b_counter
	
		else if b_param1 = READ_CH1_CH2 then
		
			for b_counter = 0 to 15
	
				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif

				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1

				readi2c IOEXP_SRAM_CH1,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
	
				readi2c IOEXP_SRAM_CH2,(b_adc_data)
	
				#ifdef HSERIAL
				hserout 0, (b_adc_data)
				#else
				sertxd (b_adc_data)
				#endif
				
				pulsout CLK_SLOW, 1
				pulsout CLK_SLOW, 1
				
			next b_counter

		endif

	endif ' b_param2 = 0 or 1
		
return


set_gain:

	b_pga_channel = b_param1
	b_pga_instruction = %01000000 ' set gain
	b_pga_data = b_param2

	gosub pga_command

	#ifdef HSERIAL
	hserout 0, (b_checksum) ' acknowledge
	#else
	sertxd (b_checksum)
	#endif
	
return


pga_command:
	' PGA channel has the channel number (1 or 2)
	' PGA_data and PGA_instrunction hold data byte and isntruction byte to send

	' enable PGA
	if b_pga_channel = 1 then
		writei2c IOEXP_PGA1_CSn,(0)
	else
		writei2c IOEXP_PGA2_CSn,(0)
	endif	

	spiout SPI_CLK, SPI_DATA, MSBFirst_L, (b_pga_instruction,b_pga_data)
	
	' latch data into PGA, enable PGA
	if b_pga_channel = 1 then
		writei2c IOEXP_PGA1_CSn,(1)
	else
		writei2c IOEXP_PGA2_CSn,(1)
	endif	
	
return


set_dac:

	b_dummy = b_param1 >> 4 ' b_dummy holds DAC channel (0..3) - DAC1 has chans 0&1, DAC2 has 2&3
	
	if b_dummy < 2 then ' DAC 1
		writei2c IOEXP_DAC1_CSn,(0)
	else ' DAC 2
		writei2c IOEXP_DAC2_CSn,(0)
	endif
	
	' strip out DAC channel, but add header bits
	if b_dummy = 0 or b_dummy = 2 then
		b_param1 = b_param1 & %00001111 | %00010000 ' DAC output 1, power up
	else
		b_param1 = b_param1 & %00001111 | %10010000 ' DAC output 2, power up
	endif	

	spiout SPI_CLK, SPI_DATA, MSBFirst_L, (b_param1, b_param2)
	
	if b_dummy < 2 then ' DAC 1
		writei2c IOEXP_DAC1_CSn,(1)
	else ' DAC 2
		writei2c IOEXP_DAC2_CSn,(1)
	endif

	#ifdef HSERIAL
	hserout 0, (b_checksum) ' acknowledge
	#else
	sertxd (b_checksum)
	#endif
	
return


set_clock_frequency:

	select b_param1

		case CLOCK_16MHz
			setfreq em16
		case CLOCK_8MHz
			setfreq m8
		case CLOCK_4MHz
			setfreq m4
		case CLOCK_2MHz
			setfreq m2
		case CLOCK_1MHz
			setfreq m1
		case CLOCK_500kHz
			setfreq k500
		case CLOCK_250kHz
			setfreq k250
		
	endselect			

return















