'AXE112 Colour module using PICAXE-08M
'Sorts coloured Smarties(TM) into correct position bin

; MODIFIED 05-08-2005 
; FOR USE WITH HITEC HS-311 SERVO 
; LIGHT SENSOR SUPPORTED BY 30mm+12mm SPACER

' ********************************************
' Servo Position Constants

' Predefined servo positions - will require editing for your setup!
' These positions need to be found by experimentation

symbol pos_tube = 203 		' tube position
symbol pos_scan = 178		' scan position

symbol pos_red = 143		' red bin
symbol pos_green = 110 		' green bin
symbol pos_blue = 83		' blue bin
symbol pos_yellow = 124		' yellow bin
symbol pos_reject = 63		' reject bin


' ********************************************
' PICAXE-08M input/output pins

symbol LED = 0	' Colour sensor white LEDs 	(output 0)
symbol S2  = 1	' Colour sensor select S2 	(output 1)
symbol ser = 2	' Servo 				(output 2)
symbol CSI = 3	' Colour sensor pulse 		(input 3)
symbol S3  = 4	' Colour sensor select S3 	(output 4)

' ********************************************
' Variables

symbol counter = b0		' Counters for loops
symbol counter2 = b1
symbol start_pos = b2		' Servo start position
symbol end_pos = b3		' Servo end position

symbol red_value	 = w4 	' Colour sensor red content
symbol blue_value	 = w5 	' Colour sensor blue content
symbol green_value = w6 	' Colour sensor green content
					' Remember w4-w6 uses b8-b13!


' ********************************************

'initialise LED and move servo to tbe tube position

init:	low LED			' make LED pin an output
	start_pos = pos_tube	' set start position
	servo ser, pos_tube	' move servo
	pause 300			' delay for servo to move

' ********************************************

' scan and sort a sweet every 3 seconds

main:
	low ser		' servo off to save power 
	pause 3000		' wait three seconds
	
' vibrate sweet out of tube

	for counter2 = 1 to 3
		end_pos = pos_tube - 2
		gosub move
		end_pos = pos_tube
		gosub move_back
	next counter2

' move to scanning position 

	end_pos = pos_scan
	gosub move

' scan the colour

	gosub colour

' move to correct bin

	gosub move
	pause 100

' back to start

	end_pos = pos_tube
	gosub move_back

	goto main

' ********************************************

' sub to move servo slowly forwards	
' moving one servo step every 18ms
' makes servo move slower than normal

move:
	for counter = start_pos to end_pos step -1
	  servo ser, counter
	  pause 18
	next counter
	start_pos = end_pos
	return

' ********************************************

' sub to move servo slowly back        

move_back:
	for counter = start_pos to end_pos  
	  servo ser, counter
	  pause 18
	next counter
	start_pos = end_pos
	return

' ********************************************

' sub to scan colours and then set the end position
' of the servo depending on which colour was identified

colour:
	high LED			' LED on

	low S2			' read red into w4
	low S3
	count 3, 50, red_value

	high S3			' read blue into w5
	count 3, 50, blue_value

	high S2			' read green into w6
	count 3, 50, green_value

	low LED			' LED off
	debug 			' optional display variables on computer screen
					' (NB: use 'word' Display Mode on debug screen)

' preload reject position
	end_pos = pos_reject	

' By experimentation it was found that the values for 
' w4, w5, and w6 need to be between these values for 
' varying colour tolerances on the sweets

'		red _value		green_value		blue_value
' blue  	0<w4<70		100<w6<300		300<w5<550
' green	0<w4<150		300<w6<500		100<w5<300
' red 	80<w4<200		20<w6<80		0<w5<100
' yellow	200<w4<350		300<w6<600		150<w5<300

symbol blue_r_min = 0
symbol blue_r_max = 70
symbol blue_g_min = 100
symbol blue_g_max = 300
symbol blue_b_min = 300
symbol blue_b_max = 550

symbol green_r_min = 0
symbol green_r_max = 150
symbol green_g_min = 300
symbol green_g_max = 500
symbol green_b_min = 100
symbol green_b_max = 3000

symbol red_r_min = 80
symbol red_r_max = 200
symbol red_g_min = 20
symbol red_g_max = 80
symbol red_b_min = 0
symbol red_b_max = 100

symbol yellow_r_min = 200
symbol yellow_r_max = 350
symbol yellow_g_min = 300
symbol yellow_g_max = 600
symbol yellow_b_min = 150
symbol yellow_b_max = 300

' now identify correct colour using these values

	' at this point can be any colour

	'if red colour the blue value must be very low - no other colour is this low 
	if blue_value > red_b_min and blue_value < red_b_max then test_red		'only can be red

	'if blue colour the blue value must be very high - no other colour is this high
	if blue_value > blue_b_min and blue_value < blue_b_max then test_blue		'only can be blue

	'if yellow colour the red value must be very high - no other colour is this high 
	'(this is strange as even red colour sweets do not have a red value this high!)
	if red_value > yellow_r_min and red_value < yellow_r_max then test_yellow	'only can be yellow

	' now the only colour left is green so start checking its red value limit
	if red_value > green_r_min and red_value < green_r_max then test_green		'could be green so more test required
	
	' only reject left now so return

is_reject:	
	return

	
' more blue tests
test_blue:
	if red_value < blue_r_min or red_value > blue_r_max then is_reject
	if green_value < blue_g_min or green_value > blue_g_max then is_reject
      'if blue_value < blue_b_min or blue_value > blue_b_max then is_reject
	end_pos = pos_blue
	return

' more red tests
test_red:
	if red_value < red_r_min or red_value > red_r_max then is_reject
	if green_value < red_g_min or green_value > red_g_max then is_reject
	'if blue_value < red_b_min or blue_value > red_b_max then is_reject
	end_pos = pos_red
	return


' more green tests
test_green:
	'if red_value < green_r_min or red_value > green_r_max then is_reject
	if green_value < green_g_min or green_value > green_g_max then is_reject
	if blue_value < green_b_min or blue_value > green_b_max then is_reject 
	end_pos = pos_green
	return


' more yellow tests
test_yellow:
	'if red_value < yellow_r_min or red_value > yellow_r_max then is_reject
	if green_value < yellow_g_min or green_value > yellow_g_max then is_reject
	if blue_value < yellow_b_min or blue_value > yellow_b_max then is_reject
	end_pos = pos_yellow
	return





















